@@ -48,12 +48,21 @@ rfbBool VncClientThread::newclientStatic(rfbClient *cl)
 }
 
 // Dispatch from this static callback context to the member context.
-void VncClientThread::updatefbStatic(rfbClient *cl, int x, int y, int w, int h)
+void VncClientThread::updatefbStaticPartial(rfbClient *cl, int x, int y, int w, int h)
 {
     VncClientThread *t = (VncClientThread *)rfbClientGetClientData(cl, nullptr);
     Q_ASSERT(t);
 
-    return t->updatefb(x, y, w, h);
+    return t->updatefbPartial(x, y, w, h);
+}
+
+// Dispatch from this static callback context to the member context.
+void VncClientThread::updateFbStaticFinished(rfbClient *cl)
+{
+    VncClientThread *t = (VncClientThread *)rfbClientGetClientData(cl, nullptr);
+    Q_ASSERT(t);
+
+    return t->updatefbFinished();
 }
 
 // Dispatch from this static callback context to the member context.
@@ -219,10 +228,15 @@ rfbBool VncClientThread::newclient()
     return true;
 }
 
-void VncClientThread::updatefb(int x, int y, int w, int h)
+void VncClientThread::updatefbPartial(int x, int y, int w, int h)
 {
-//     qCDebug(KRDC) << "updated client: x: " << x << ", y: " << y << ", w: " << w << ", h: " << h;
+//    qCDebug(KRDC) << "updated client: x: " << x << ", y: " << y << ", w: " << w << ", h: " << h;
 
+    m_dirtyRect = m_dirtyRect.united(QRect(x, y, w, h));
+}
+
+void VncClientThread::updatefbFinished()
+{
     const int width = cl->width, height = cl->height;
     QImage img;
     switch(colorDepth()) {
@@ -249,12 +263,17 @@ void VncClientThread::updatefb(int x, int y, int w, int h)
     img.setDevicePixelRatio(m_devicePixelRatio);
     setImage(img);
 
-    emitUpdated(x, y, w, h);
+    // Compute bounding rect.
+    QRect updateRect = m_dirtyRect;
+    m_dirtyRect = QRect();
+
+//    qCDebug(KRDC) << Q_FUNC_INFO << updateRect;
+    emitUpdated(updateRect.x(), updateRect.y(), updateRect.width(), updateRect.height());
 }
 
 void VncClientThread::cuttext(const char *text, int textlen)
 {
-    const QString cutText = QString::fromUtf8(text, textlen);
+    const QString cutText = QString::fromLatin1(text, textlen);
     qCDebug(KRDC) << cutText;
 
     if (!cutText.isEmpty()) {
@@ -591,7 +610,8 @@ bool VncClientThread::clientCreate(bool reinitialising)
     cl->canHandleNewFBSize = true;
     cl->GetPassword = passwdHandlerStatic;
     cl->GetCredential = credentialHandlerStatic;
-    cl->GotFrameBufferUpdate = updatefbStatic;
+    cl->GotFrameBufferUpdate = updatefbStaticPartial;
+    cl->FinishedFrameBufferUpdate = updateFbStaticFinished;
     cl->GotXCutText = cuttextStatic;
     cl->GotCursorShape = cursorShapeHandlerStatic;
     rfbClientSetClientData(cl, nullptr, this);
@@ -709,8 +729,8 @@ void KeyClientEvent::fire(rfbClient* cl)
 
 void ClientCutEvent::fire(rfbClient* cl)
 {
-    QByteArray toUtf8Converted = text.toUtf8();
-    SendClientCutText(cl, toUtf8Converted.data(), toUtf8Converted.length());
+    QByteArray toLatin1Converted = text.toLatin1();
+    SendClientCutText(cl, toLatin1Converted.data(), toLatin1Converted.length());
 }
 
 void VncClientThread::mouseEvent(int x, int y, int buttonMask)
